#include "app_env.h"
#include "nrf_temp.h"
#include "app/app_motion.h"
#include <Arduino.h>

static void _nrf_temperature_init(void);
static int32_t _nrf_temperature_get(void);

static void _bma_temperature_init(void);
static int32_t _bma_temperature_get(void);

void app_env_init(void)
{
    _nrf_temperature_init(); //会导致重启
}

int32_t app_env_get_temperature(void)
{
    return _nrf_temperature_get();
}

static void _nrf_temperature_init(void)
{
    // Serial.println("_temperature_init");
    nrf_temp_init();
}

static int32_t _nrf_temperature_get(void)
{
    int32_t volatile temp;
    NRF_TEMP->TASKS_START = 1; /** Start the temperature measurement. */

    /* Busy wait while temperature measurement is not finished, you can skip waiting if you enable interrupt for DATARDY event and read the result in the interrupt. */
    /*lint -e{845} // A zero has been given as right argument to operator '|'" */
    while (NRF_TEMP->EVENTS_DATARDY == 0)
    {
        // Do nothing.
        // Serial.print(". ");
        delay(10);
    }
    NRF_TEMP->EVENTS_DATARDY = 0;

    /**@note Workaround for PAN_028 rev2.0A anomaly 29 - TEMP: Stop task clears the TEMP register. */
    temp = (nrf_temp_read() / 4);

    /**@note Workaround for PAN_028 rev2.0A anomaly 30 - TEMP: Temp module analog front end does not power down when DATARDY event occurs. */
    NRF_TEMP->TASKS_STOP = 1; /** Stop the temperature measurement. */
    return temp;
}
