#include "app_key.h"

#include "Button2.h"
#include <Arduino.h>

#include "app_motor.h"

QueueHandle_t Key_Queue;
BaseType_t err;

Button2 key0 = Button2(PIN_KEY0, INPUT, true, 10);
Button2 key1 = Button2(PIN_KEY1, INPUT, true, 10);

SoftwareTimer keyTimer;
SoftwareTimer longTimer; //长按提示

extern SoftwareTimer timeoutTimer;

static KEY_TYPE key_type;

static void handler(Button2 &btn);
static void key_callback(void);

/////////////////////////////////////////////////////////////////

static void pressed(Button2 &btn)
{
  // Serial.println("pressed");
  if (btn == key0)
  {
    key_type = KEY0_DOWN;
  }
  else if (btn == key1)
  {
    key_type = KEY1_DOWN;
  }
  // app_motor_once(60);
  timeoutTimer.reset();
  longTimer.start();

  if (Key_Queue != 0)
  {
    err = xQueueSend(Key_Queue, &key_type, 10);
    if (err = errQUEUE_FULL)
    {
      // 队列已满，数据发送失败
    }
  }
}
static void released(Button2 &btn)
{
  // Serial.print("released: ");
  // Serial.println(btn.wasPressedFor());

  if (btn == key0)
  {
    key_type = KEY0_UP;
  }
  else if (btn == key1)
  {
    key_type = KEY1_UP;
  }
  longTimer.stop();

  if (Key_Queue != 0)
  {
    err = xQueueSend(Key_Queue, &key_type, 10);
    if (err = errQUEUE_FULL)
    {
      // 队列已满，数据发送失败
    }
  }
}
static void changed(Button2 &btn)
{
  // Serial.println("changed");
}
static void click(Button2 &btn)
{
  // Serial.println("click\n");
  if (btn == key0)
  {
    key_type = KEY0_SHORT;
  }
  else if (btn == key1)
  {
    key_type = KEY1_SHORT;
  }

  if (Key_Queue != 0)
  {
    err = xQueueSend(Key_Queue, &key_type, 10);
    if (err = errQUEUE_FULL)
    {
      // 队列已满，数据发送失败
    }
  }
}
static void longClick(Button2 &btn)
{
  // Serial.println("long click\n");
  if (btn == key0)
  {
    key_type = KEY0_LONG;
  }
  else if (btn == key1)
  {
    key_type = KEY1_LONG;
  }

  if (Key_Queue != 0)
  {
    err = xQueueSend(Key_Queue, &key_type, 10);
    if (err = errQUEUE_FULL)
    {
      // 队列已满，数据发送失败
    }
  }
}
static void doubleClick(Button2 &btn)
{
  // Serial.println("double click\n");
}
static void tripleClick(Button2 &btn)
{
  // Serial.println("triple click\n");
}
static void tap(Button2 &btn)
{
  // Serial.println("tap");
}
/////////////////////////////////////////////////////////////////

static void key_timer_callback(TimerHandle_t xTimerID)
{
  // freeRTOS timer ID, ignored if not used
  (void)xTimerID;
  key0.loop();
  key1.loop();
}

static void long_timer_callback(TimerHandle_t xTimerID)
{
  // freeRTOS timer ID, ignored if not used
  (void)xTimerID;
  app_motor_once(70);
}

void app_key_init(void)
{

  pinMode(PIN_KEY0, INPUT);
  pinMode(PIN_KEY1, INPUT);

  // key0.setChangedHandler(changed);
  key0.setPressedHandler(pressed);
  key0.setReleasedHandler(released);

  // key0.setTapHandler(tap);
  key0.setClickHandler(click);
  key0.setLongClickHandler(longClick);
  // key0.setDoubleClickHandler(doubleClick);
  // key0.setTripleClickHandler(tripleClick);

  // key1.setChangedHandler(changed);
  key1.setPressedHandler(pressed);
  key1.setReleasedHandler(released);

  // key1.setTapHandler(tap);
  key1.setClickHandler(click);
  key1.setLongClickHandler(longClick);
  // key1.setDoubleClickHandler(doubleClick);
  // key1.setTripleClickHandler(tripleClick);

  // ISR_DEFERRED flag cause the callback to be deferred from ISR context
  // and invoked within a callback thread.
  // It is required to use ISR_DEFERRED if callback function take long time
  // to run e.g Serial.print() or using any of Bluefruit API() which will
  // potentially call rtos API
  //   attachInterrupt(interruptPin, digital_callback, ISR_DEFERRED | CHANGE);
  // attachInterrupt(PIN_KEY0, key_callback,  CHANGE);

  //创建按键消息队列
  Key_Queue = xQueueCreate(10, sizeof(uint8_t));
  if (Key_Queue == 0)
  {
    // Queue was not created and must not be used.
  }

  // tpkey.setLongClickHandler(handler);
  // // Configure the timer with 1000 ms interval, with our callback
  keyTimer.begin(10, key_timer_callback);

  // // Start the timer
  if (!keyTimer.start())
  {
    // tft.println("timer start fail");
  }

  // tpkey.setLongClickHandler(handler);
  // // Configure the timer with 1000 ms interval, with our callback
  longTimer.begin(1000, long_timer_callback, NULL, false);
}

void app_key_deinit(void)
{

  keyTimer.~SoftwareTimer();

  nrf_gpio_cfg_default(PIN_KEY0);
  nrf_gpio_cfg_default(PIN_KEY1);
}

void app_key_config_wakeup(void)
{
  pinMode(PIN_KEY0, INPUT_PULLUP_SENSE); // this pin (WAKE_HIGH_PIN) is pulled down and wakes up the feather when externally connected to 3.3v.
  pinMode(PIN_KEY1, INPUT_PULLUP_SENSE); // this pin (WAKE_HIGH_PIN) is pulled down and wakes up the feather when externally connected to 3.3v.
}

KEY_TYPE app_key_get()
{
  key_type = KEY_NONE;
  if (Key_Queue != 0)
  {
    // if(xQueueReceive(Key_Queue, &key_type,portMAX_DELAY))    //  10/portTICK_PERIOD_MS
    if (xQueueReceive(Key_Queue, &key_type, 0))
    {
      return key_type;
    }
  }

  return key_type;
}

//清楚多余按键信息
void app_key_clean()
{
  if (Key_Queue != 0)
  {
    xQueueReset(Key_Queue);
  }
}
// static void handler(Button2& btn) {

//     if (btn == key0)
//     {
//       Serial.println("A clicked");
//     }
//     else if (btn == key1)
//     {
//       Serial.println("B clicked");
//     }

//   static uint8_t i =0;
//      unsigned int time = btn.wasPressedFor();
//     if(time > 8000)
//     {
//         NVIC_SystemReset();

//     }
//     else if (time > 1000 &&time<2000)
//     {
//         //long press
//         key_type = KEY_LONG;
//     }
//     else if(time >= 100 && time<400)
//     {
//         key_type = KEY_SHORT;
//     }
//     else
//     {
//         key_type = KEY_NONE;
//     }

//     // tft.print(" (");
//     // tft.print(time);
//     // tft.println(" ms)");
// }
// bool pressed = false;
// static void key_callback(void)
// {
//     static uint32_t t1,t2;
//   if(digitalRead(PIN_KEY0))
//   {
//       pressed = true;
//       t1=millis();

//   }
//   else
//   {
//       t2 =millis()-t1;
//       if((t2)>100 && t2<=500)
//       {
//           key_type = KEY_SHORT;
//       }
//       else if(t2>1000 && t2<2000)
//       {
//           key_type = KEY_LONG;
//       }
//       else
//       {
//           key_type = KEY_NONE;
//       }

//       pressed = false;
//   }
// }
