#include "game_select.h"
#include "page/page_manager.h"

#include "app/app_anim.h"
#include "app/app_key.h"

#include "page/game/flappybird/flappybird.h"

static const char *name[] = {"FlappyBird", "Dino", "Snake"};
#define ITEM_NUM 3
// #define ITEM_NUM  sizeof(name)

static int8_t item = 0;
static lv_obj_t *list;
static lv_obj_t *list_btn[ITEM_NUM];

// static void event_handler(lv_obj_t * obj, lv_event_t event);
static void anim_in_ready_cb(lv_anim_t *a)
{
  app_key_clean();
  manager_set_busy_state(false);
}

static void anim_out_ready_cb(lv_anim_t *a)
{
  lv_obj_del(list);
}

lv_obj_t *game_select_enter(void *data)
{

  /*Create a list*/
  list = lv_list_create(lv_scr_act(), NULL);
  lv_obj_set_size(list, LV_HOR_RES, LV_VER_RES);
  lv_obj_align(list, NULL, LV_ALIGN_CENTER, 0, 0);
  lv_list_set_edge_flash(list, true);
  lv_list_set_anim_time(list, 300);

  lv_obj_set_style_local_bg_color(list, LV_LIST_PART_BG, LV_STATE_DEFAULT, LV_COLOR_BLACK);
  lv_obj_set_style_local_pad_left(list, LV_LIST_PART_BG, LV_STATE_DEFAULT, 0);
  lv_obj_set_style_local_pad_bottom(list, LV_LIST_PART_BG, LV_STATE_DEFAULT, 0);
  lv_obj_set_style_local_pad_top(list, LV_LIST_PART_BG, LV_STATE_DEFAULT, 0);
  lv_obj_set_style_local_pad_right(list, LV_LIST_PART_BG, LV_STATE_DEFAULT, 0);
  lv_obj_set_style_local_border_opa(list, LV_LIST_PART_BG, LV_STATE_DEFAULT, LV_OPA_TRANSP);
  lv_obj_set_style_local_border_width(list, LV_LIST_PART_BG, LV_STATE_DEFAULT, 0);
  lv_obj_set_style_local_outline_opa(list, LV_LIST_PART_BG, LV_STATE_DEFAULT, LV_OPA_TRANSP);

  /*Add buttons to the list*/

  for (uint8_t i = 0; i < ITEM_NUM; i++)
  {
    list_btn[i] = lv_list_add_btn(list, NULL, name[i]);
    lv_list_focus(list_btn[i], LV_ANIM_ON);

    // lv_obj_set_event_cb(list_btn[i], event_handler);
    lv_obj_set_style_local_bg_color(list_btn[i], LV_BTN_PART_MAIN, LV_STATE_FOCUSED, LV_COLOR_MAKE(0x00, 0x66, 0x99)); //蓝色
    lv_obj_set_style_local_outline_opa(list_btn[i], LV_BTN_PART_MAIN, LV_STATE_FOCUSED, LV_OPA_TRANSP);
    lv_obj_set_style_local_text_font(list_btn[i], LV_BTN_PART_MAIN, LV_STATE_FOCUSED, &lv_font_unscii_8);
    lv_obj_set_style_local_text_color(list_btn[i], LV_BTN_PART_MAIN, LV_STATE_FOCUSED, LV_COLOR_BLACK);

    // lv_obj_set_style_local_bg_opa(list_btn[i],LV_BTN_PART_MAIN,LV_STATE_DEFAULT,LV_OPA_TRANSP);
    lv_obj_set_style_local_bg_color(list_btn[i], LV_BTN_PART_MAIN, LV_STATE_DEFAULT, LV_COLOR_BLACK);
    lv_obj_set_style_local_text_font(list_btn[i], LV_BTN_PART_MAIN, LV_STATE_DEFAULT, &lv_font_unscii_8);
    lv_obj_set_style_local_outline_width(list_btn[i], LV_BTN_PART_MAIN, LV_STATE_DEFAULT, 0);
    lv_obj_set_style_local_border_width(list_btn[i], LV_BTN_PART_MAIN, LV_STATE_DEFAULT, 0);
    lv_obj_set_style_local_outline_opa(list_btn[i], LV_BTN_PART_MAIN, LV_STATE_DEFAULT, LV_OPA_TRANSP);
  }

  lv_list_focus_btn(list, list_btn[item]);

  // lv_mem_monitor_t info;
  // lv_mem_monitor(&info);
  // lv_obj_t *mem = lv_label_create(list, NULL);
  // lv_obj_set_style_local_text_font(mem, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, &lv_font_unscii_8);
  // lv_obj_set_style_local_text_color(mem, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, LV_COLOR_RED);
  // lv_label_set_text_fmt(mem, "%d", info.free_size);
  // lv_obj_align(mem, NULL, LV_ALIGN_IN_BOTTOM_RIGHT, 0, 0);

  //进入动画，进行切换
  obj_anim_in(list, 200, 200, anim_in_ready_cb, NULL);

  return list;
}

//根据按键情况，对其进行不同操作处理
void game_select_loop(void *data)
{
  KEY_TYPE key = app_key_get();
  switch (key)
  {
  case KEY0_SHORT:
    if (list != NULL)
    {

      lv_list_down(list);
      item--;
      if (item <= 0)
        item = 0;
      lv_list_focus_btn(list, list_btn[item]);
      //  lv_list_focus(list_btn[item], LV_ANIM_ON);
    }

    break;
  case KEY1_SHORT:
    if (list != NULL)
    {
      lv_list_up(list);
      item++;
      if (item >= ITEM_NUM)
        item = ITEM_NUM - 1;

      lv_list_focus_btn(list, list_btn[item]);
      // lv_list_focus(list_btn[item], LV_ANIM_ON);
    }
    break;
  case KEY0_LONG:
    item = 0;
    manager_switch(LEVEL_PARENT);
    break;
  case KEY1_LONG:
    manager_switch_child(item);
    //   manager_switch(LEVEL_CHILD);
    // if (list != NULL)
    // {
    //   if (lv_list_get_btn_index(list, lv_list_get_btn_selected(list)) == 0)
    //   {

    //     flappybird_enter(list);
    //   }
    // }

    break;

  default:
    break;
  }
}

void game_select_exit(void *data)
{

  // lv_task_del(task);
  //退出动画，进行切换
  obj_anim_out(list, 100, 0, anim_out_ready_cb, NULL);
}
