#include "page_about.h"

#include <Arduino.h>

#include "board/board.h"
#include "app/app_key.h"
#include "app/app_anim.h"
#include "app/app_hwinfo.h"
#include "page/page_manager.h"
static lv_obj_t *scr;

static void anim_in_ready_cb(lv_anim_t *a)
{
    app_key_clean();
    manager_set_busy_state(false);
}

static void anim_out_ready_cb(lv_anim_t *a)
{
    lv_obj_del(scr);
}

lv_obj_t *page_about_enter(void *data)
{

    scr = lv_obj_create(lv_scr_act(), NULL);
    lv_obj_clean_style_list(scr, LV_OBJ_PART_MAIN);
    lv_obj_set_style_local_bg_opa(scr, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, LV_OPA_COVER);
    lv_obj_set_style_local_bg_color(scr, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, LV_COLOR_BLACK);
    lv_obj_set_style_local_border_color(scr, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, LV_COLOR_BLACK);
    lv_obj_set_style_local_radius(scr, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, 0);
    lv_obj_set_size(scr, LV_HOR_RES, LV_VER_RES);
    lv_obj_set_pos(scr, 0, 0);

    lv_obj_t *sw = lv_label_create(scr, NULL);
    lv_obj_set_style_local_text_font(sw, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, &lv_font_unscii_8);
    lv_obj_set_style_local_text_color(sw, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, lv_color_hex(0x7acd32)); //
    lv_label_set_text_fmt(sw, "SW:v%d.%d.%d %s", SW_VERSION_MAJOR, SW_VERSION_MINOR, SW_VERSION_PATCH, SW_VERSION_INFO);
    lv_obj_align(sw, NULL, LV_ALIGN_IN_TOP_LEFT, 0, 5);

    lv_obj_t *hw = lv_label_create(scr, NULL);
    lv_obj_set_style_local_text_font(hw, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, &lv_font_unscii_8);
    lv_obj_set_style_local_text_color(hw, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, lv_color_hex(0x8acd32)); //
    lv_label_set_text_fmt(hw, "HW:v%d.%d.%d %s", HW_VERSION_MAJOR, HW_VERSION_MINOR, HW_VERSION_PATCH, HW_VERSION_INFO);
    lv_obj_align(hw, sw, LV_ALIGN_OUT_BOTTOM_LEFT, 0, 5);

    char buffer_mac[20];
    app_hwinfo_get_mac(buffer_mac);

    lv_obj_t *mac = lv_label_create(scr, NULL);
    lv_obj_set_style_local_text_font(mac, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, &lv_font_unscii_8);
    lv_obj_set_style_local_text_color(mac, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, lv_color_hex(0x9acd32)); //yellowgreen
    lv_label_set_text_fmt(mac, "MAC:%s", buffer_mac);
    lv_label_set_long_mode(mac, LV_LABEL_LONG_SROLL);
    lv_obj_set_width(mac, LV_HOR_RES);

    char buffer_id[20];
    app_hwinfo_get_chipid(buffer_id);
    lv_obj_t *id = lv_label_create(scr, NULL);
    lv_obj_set_style_local_text_font(id, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, &lv_font_unscii_8);
    lv_obj_set_style_local_text_color(id, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, lv_color_hex(0xaacd32)); //yellowgreen
    lv_label_set_text_fmt(id, "ID:%s", buffer_id);

    lv_obj_t *designer = lv_label_create(scr, NULL);
    lv_obj_set_style_local_text_font(designer, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, &lv_font_unscii_8);
    lv_obj_set_style_local_text_color(designer, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, lv_color_hex(0xbacd32)); //yellowgreen
    lv_label_set_text(designer, "Designer:Modular");

    lv_obj_align(mac, hw, LV_ALIGN_OUT_BOTTOM_LEFT, 0, 5);
    lv_obj_align(id, mac, LV_ALIGN_OUT_BOTTOM_LEFT, 0, 5);
    lv_obj_align(designer, id, LV_ALIGN_OUT_BOTTOM_LEFT, 0, 5);

    // lv_mem_monitor_t info;
    // lv_mem_monitor(&info);
    // lv_obj_t *mem = lv_label_create(scr, NULL);
    // lv_obj_set_style_local_text_font(mem, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, &lv_font_unscii_8);
    // lv_obj_set_style_local_text_color(mem, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, LV_COLOR_RED);
    // lv_label_set_text_fmt(mem, "%d", info.free_size);
    // lv_obj_align(mem, NULL, LV_ALIGN_IN_BOTTOM_RIGHT, 0, 0);

    obj_anim_in(sw, 100, 200, NULL, NULL);
    obj_anim_in(hw, 100, 300, NULL, NULL);
    obj_anim_in(mac, 100, 400, NULL, NULL);
    obj_anim_in(id, 100, 500, NULL, NULL);
    obj_anim_in(designer, 100, 600, NULL, NULL);

    //进入动画，进行切换
    obj_anim_in(scr, 100, 100, anim_in_ready_cb, NULL);
    return scr;
}

//根据按键情况，对其进行不同操作处理
void page_about_loop(void *data)
{
    KEY_TYPE key = app_key_get();

    switch (key)
    {
    case KEY0_SHORT:
        //   manager_switch(LEVEL_PREV);
        break;
    case KEY1_SHORT:
        //   manager_switch(LEVEL_NEXT);

        break;
    case KEY0_LONG:
        manager_switch(LEVEL_PARENT);
        break;
    case KEY1_LONG:
        // manager_switch(LEVEL_CHILD);
        break;

    default:
        break;
    }
}

void page_about_exit(void *data)
{

    // lv_task_del(task);
    //退出动画，进行切换
    obj_anim_out(scr, 100, 0, anim_out_ready_cb, NULL);
}
