#include "page_backlight.h"

#include <Arduino.h>

#include "app/app_key.h"
#include "app/app_anim.h"
#include "app/app_backlight.h"
#include "page/page_manager.h"
static lv_obj_t *scr;
static lv_obj_t *slider;
static lv_obj_t *title;

static void anim_in_ready_cb(lv_anim_t *a)
{
    app_key_clean();
    manager_set_busy_state(false);
}

static void anim_out_ready_cb(lv_anim_t *a)
{
    lv_obj_del(scr);
}

lv_obj_t *page_backlight_enter(void *data)
{

    uint8_t level = app_backlight_get_level();

    scr = lv_obj_create(lv_scr_act(), NULL);
    lv_obj_clean_style_list(scr, LV_OBJ_PART_MAIN);
    lv_obj_set_style_local_bg_opa(scr, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, LV_OPA_COVER);
    lv_obj_set_style_local_bg_color(scr, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, LV_COLOR_BLACK);
    lv_obj_set_style_local_border_color(scr, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, LV_COLOR_BLACK);
    lv_obj_set_style_local_radius(scr, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, 0);
    lv_obj_set_size(scr, LV_HOR_RES, LV_VER_RES);
    lv_obj_set_pos(scr, 0, 0);

    /* Create a slider in the center of the display */
    slider = lv_slider_create(scr, NULL);
    lv_obj_set_width(slider, LV_HOR_RES - 30);
    lv_obj_align(slider, NULL, LV_ALIGN_CENTER, 0, -10);
    lv_slider_set_range(slider, 1, 10);
    lv_slider_set_value(slider, level, true);

    title = lv_label_create(scr, NULL);
    //  lv_obj_set_style_local_text_font(title,LV_OBJ_PART_MAIN,LV_STATE_DEFAULT,&thin64Font);
    //  lv_obj_set_style_local_text_color(title,LV_OBJ_PART_MAIN,LV_STATE_DEFAULT,lv_color_hex(0x9acd32));//yellowgreen
    lv_label_set_text_fmt(title, "Bright:%2d", level);
    lv_obj_align(title, slider, LV_ALIGN_OUT_BOTTOM_MID, 0, 10);

    // lv_mem_monitor_t info;
    // lv_mem_monitor(&info);
    // lv_obj_t *mem = lv_label_create(scr, NULL);
    // lv_obj_set_style_local_text_font(mem, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, &lv_font_unscii_8);
    // lv_obj_set_style_local_text_color(mem, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, LV_COLOR_RED);
    // lv_label_set_text_fmt(mem, "%d", info.free_size);
    // lv_obj_align(mem, NULL, LV_ALIGN_IN_BOTTOM_RIGHT, 0, 0);

    //进入动画，进行切换
    obj_anim_in(scr, 100, 100, anim_in_ready_cb, NULL);
    return scr;
}

//根据按键情况，对其进行不同操作处理
void page_backlight_loop(void *data)
{
    KEY_TYPE key = app_key_get();
    uint8_t level = app_backlight_get_level();
    switch (key)
    {
    case KEY0_SHORT:
        //   manager_switch(LEVEL_PREV);
        level--;
        if (level <= 1)
        {
            level = 1;
        }
        app_backlight_set_level(level);
        lv_label_set_text_fmt(title, "Bright:%2d", level);
        lv_slider_set_value(slider, level, true);

        break;
    case KEY1_SHORT:
        //   manager_switch(LEVEL_NEXT);
        level++;
        if (level >= 10)
        {
            level = 10;
        }
        app_backlight_set_level(level);
        lv_label_set_text_fmt(title, "Bright:%2d", level);
        lv_slider_set_value(slider, level, true);
        break;
    case KEY0_LONG:
        manager_switch(LEVEL_PARENT);
        break;
    case KEY1_LONG:
        // manager_switch(LEVEL_CHILD);
        break;

    default:
        break;
    }
}

void page_backlight_exit(void *data)
{

    // lv_task_del(task);
    //退出动画，进行切换
    obj_anim_out(scr, 100, 0, anim_out_ready_cb, NULL);
}
