#include "page_calendar.h"
#include "app/app_batt.h"
#include "app/app_anim.h"
#include "app/app_key.h"
#include "page/page_manager.h"

extern PCF8563_Class rtc;

static lv_obj_t *scr;
static lv_obj_t *calendar;

static void anim_in_ready_cb(lv_anim_t *a);
static void anim_out_ready_cb(lv_anim_t *a);

lv_obj_t *page_calendar_enter(void *data)
{
  scr = lv_obj_create(lv_scr_act(), NULL);
  lv_obj_clean_style_list(scr, LV_OBJ_PART_MAIN);
  lv_obj_set_style_local_bg_opa(scr, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, LV_OPA_COVER);
  lv_obj_set_style_local_bg_color(scr, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, LV_COLOR_BLACK);
  lv_obj_set_style_local_border_color(scr, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, LV_COLOR_BLACK);
  lv_obj_set_style_local_radius(scr, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, 0);
  lv_obj_set_size(scr, LV_HOR_RES, LV_VER_RES);
  lv_obj_set_pos(scr, 0, 0);

  calendar = lv_calendar_create(scr, NULL);
  lv_obj_set_size(calendar, LV_HOR_RES, 80);
  // lv_obj_set_width(calendar, LV_HOR_RES);
  lv_obj_set_style_local_bg_color(calendar, LV_CALENDAR_PART_BG, LV_STATE_DEFAULT, LV_COLOR_BLACK);
  lv_obj_set_style_local_border_width(calendar, LV_CALENDAR_PART_BG, LV_STATE_DEFAULT, 0);
  lv_obj_set_style_local_text_font(calendar, LV_CALENDAR_PART_BG, LV_STATE_DEFAULT, &lv_font_unscii_8);

  lv_obj_set_style_local_pad_bottom(calendar, LV_CALENDAR_PART_DATE, LV_STATE_DEFAULT, 0);
  lv_obj_set_style_local_pad_top(calendar, LV_CALENDAR_PART_DATE, LV_STATE_DEFAULT, 0);

  lv_obj_set_style_local_pad_bottom(calendar, LV_CALENDAR_PART_DATE, LV_STATE_FOCUSED, 0);
  lv_obj_set_style_local_pad_top(calendar, LV_CALENDAR_PART_DATE, LV_STATE_FOCUSED, 0);

  lv_obj_set_style_local_pad_bottom(calendar, LV_CALENDAR_PART_DATE, LV_STATE_CHECKED, 0);
  lv_obj_set_style_local_pad_top(calendar, LV_CALENDAR_PART_DATE, LV_STATE_CHECKED, 0);

  // lv_obj_set_style_local_pad_bottom(calendar, LV_CALENDAR_PART_DATE, LV_STATE_HOVERED,0);
  // lv_obj_set_style_local_pad_top(calendar, LV_CALENDAR_PART_DATE, LV_STATE_HOVERED,0);

  // lv_obj_set_style_local_pad_left(calendar, LV_CALENDAR_PART_DATE, LV_STATE_DEFAULT,5);
  // lv_obj_set_style_local_pad_right(calendar, LV_CALENDAR_PART_DATE, LV_STATE_DEFAULT,5);

  lv_obj_set_style_local_pad_bottom(calendar, LV_CALENDAR_PART_DAY_NAMES, LV_STATE_DEFAULT, 0);
  lv_obj_set_style_local_pad_top(calendar, LV_CALENDAR_PART_DAY_NAMES, LV_STATE_DEFAULT, 0);

  lv_obj_set_style_local_pad_bottom(calendar, LV_CALENDAR_PART_HEADER, LV_STATE_DEFAULT, 0);
  lv_obj_set_style_local_pad_top(calendar, LV_CALENDAR_PART_HEADER, LV_STATE_DEFAULT, 0);

  lv_obj_set_style_local_text_font(calendar, LV_CALENDAR_PART_HEADER, LV_STATE_DEFAULT, &lv_font_unscii_8);
  lv_obj_set_style_local_text_font(calendar, LV_CALENDAR_PART_DAY_NAMES, LV_STATE_DEFAULT, &lv_font_unscii_8);

  // lv_obj_set_style_local_text_color(calendar, LV_CALENDAR_PART_DATE, LV_STATE_DEFAULT,LV_COLOR_GRAY);
  // lv_obj_set_style_local_text_color(calendar, LV_CALENDAR_PART_DATE, LV_STATE_FOCUSED,LV_COLOR_GREEN);
  lv_obj_set_style_local_text_font(calendar, LV_CALENDAR_PART_DATE, LV_STATE_DEFAULT, &lv_font_unscii_8);
  // lv_obj_set_style_local_margin_top(calendar, LV_CALENDAR_PART_DATE, LV_STATE_FOCUSED, 0);
  // lv_obj_set_style_local_margin_bottom(calendar, LV_CALENDAR_PART_DATE, LV_STATE_FOCUSED, 0);

  // lv_obj_set_style_local_margin_top(calendar, LV_CALENDAR_PART_DATE, LV_STATE_DEFAULT, 0);
  // lv_obj_set_style_local_margin_bottom(calendar, LV_CALENDAR_PART_DATE, LV_STATE_DEFAULT, 0);

  lv_obj_set_style_local_pad_inner(calendar, LV_CALENDAR_PART_DATE, LV_STATE_DEFAULT, 0);
  lv_obj_set_style_local_pad_inner(calendar, LV_CALENDAR_PART_DATE, LV_STATE_FOCUSED, 0);

  lv_mem_monitor_t info;
  lv_mem_monitor(&info);
  lv_obj_t *mem = lv_label_create(scr, NULL);
  lv_obj_set_style_local_text_font(mem, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, &lv_font_unscii_8);
  lv_obj_set_style_local_text_color(mem, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, LV_COLOR_RED);
  lv_label_set_text_fmt(mem, "%d", info.free_size);
  lv_obj_align(mem, NULL, LV_ALIGN_IN_BOTTOM_RIGHT, 0, 0);

  //进入动画，进行切换
  obj_anim_in(scr, 100, 100, anim_in_ready_cb, NULL);
  return scr;
}

static void anim_in_ready_cb(lv_anim_t *a)
{
  app_key_clean();
  manager_set_busy_state(false);
}

static void anim_out_ready_cb(lv_anim_t *a)
{
  lv_obj_del(scr);
}

//根据按键情况，对其进行不同操作处理
void page_calendar_loop(void *data)
{
  KEY_TYPE key = app_key_get();
  switch (key)
  {
  case KEY0_SHORT:
    // manager_switch(LEVEL_PREV);
    if (calendar != NULL)
    {
    }
    break;
  case KEY1_SHORT:
    // manager_switch(LEVEL_NEXT);
    break;
  case KEY0_LONG:
    manager_switch(LEVEL_PARENT);
    break;
  case KEY1_LONG:
    // manager_switch(LEVEL_CHILD);
    break;

  default:
    break;
  }
}

void page_calendar_exit(void *data)
{

  // lv_task_del(task);
  //退出动画，进行切换
  obj_anim_out(scr, 100, 0, anim_out_ready_cb, NULL);
}