#include "page_clock.h"
#include "app/app_batt.h"
#include "app/app_anim.h"
#include "app/app_key.h"
LV_FONT_DECLARE(thin64Font);

extern PCF8563_Class rtc;
extern RTC_Date date;
static lv_obj_t *scr;
static lv_obj_t *label_batt;
static lv_obj_t *label_vol;
static lv_obj_t *label_time;
static lv_obj_t *label_date;
static lv_obj_t *led_second;

static lv_task_t *task;
static void anim_in_ready_cb(lv_anim_t *a);
static void anim_out_ready_cb(lv_anim_t *a);

static void task_time_update(lv_task_t *t);

lv_obj_t *page_clock_enter(void *data)
{

  date = rtc.getDateTime();
  RTC_Date *pdate = &date;
  scr = lv_obj_create(lv_scr_act(), NULL);
  lv_obj_clean_style_list(scr, LV_OBJ_PART_MAIN);
  lv_obj_set_style_local_bg_opa(scr, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, LV_OPA_COVER);
  lv_obj_set_style_local_bg_color(scr, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, LV_COLOR_BLACK);
  lv_obj_set_style_local_border_color(scr, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, LV_COLOR_BLACK);
  lv_obj_set_style_local_radius(scr, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, 0);
  lv_obj_set_size(scr, LV_HOR_RES, LV_VER_RES);
  lv_obj_set_pos(scr, 0, 0);

  label_time = lv_label_create(scr, NULL);
  lv_obj_set_style_local_text_font(label_time, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, &thin64Font);
  lv_obj_set_style_local_text_color(label_time, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, lv_color_hex(0x9acd32)); //yellowgreen

  lv_label_set_text_fmt(label_time, "%02d%02d", pdate->hour, pdate->minute);
  lv_obj_align(label_time, scr, LV_ALIGN_IN_TOP_MID, 0, -10);

  label_date = lv_label_create(scr, NULL);
  lv_obj_set_style_local_text_font(label_date, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, &lv_font_unscii_8);
  lv_label_set_text_fmt(label_date, "%04d-%02d-%02d", pdate->year, pdate->month, pdate->day);
  lv_obj_align(label_date, scr, LV_ALIGN_IN_BOTTOM_MID, 0, 0);

  label_batt = lv_label_create(scr, NULL);
  lv_obj_set_style_local_text_font(label_batt, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, &lv_font_unscii_8);
  lv_label_set_text_fmt(label_batt, "%d%%", app_batt_percent());
  lv_obj_align(label_batt, NULL, LV_ALIGN_IN_BOTTOM_LEFT, 0, 0);

  led_second = lv_led_create(scr, NULL);
  lv_led_set_bright(led_second, LV_LED_BRIGHT_MIN);
  lv_obj_set_style_local_bg_color(led_second, LV_LED_PART_MAIN, LV_STATE_DEFAULT, LV_COLOR_GREEN);
  lv_obj_set_style_local_shadow_color(led_second, LV_LED_PART_MAIN, LV_STATE_DEFAULT, LV_COLOR_GRAY);
  lv_obj_set_style_local_border_color(led_second, LV_LED_PART_MAIN, LV_STATE_DEFAULT, LV_COLOR_GRAY);
  lv_obj_set_style_local_bg_grad_color(led_second, LV_LED_PART_MAIN, LV_STATE_DEFAULT, LV_COLOR_GRAY);
  lv_obj_set_size(led_second, 10, 10);
  lv_obj_align(led_second, NULL, LV_ALIGN_IN_TOP_RIGHT, 0, 0);

  task = lv_task_create(task_time_update, 500, LV_TASK_PRIO_MID, NULL);

  label_vol = lv_label_create(scr, NULL);
  lv_obj_set_style_local_text_font(label_vol, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, &lv_font_unscii_8);
  // lv_obj_set_style_local_text_color(label_vol,LV_OBJ_PART_MAIN,LV_STATE_DEFAULT,LV_COLOR_RED);
  lv_label_set_text_fmt(label_vol, "%d", app_batt_volatge());
  lv_obj_align(label_vol, NULL, LV_ALIGN_IN_BOTTOM_RIGHT, 0, 0);

  // lv_mem_monitor_t info;
  // lv_mem_monitor(&info);
  // lv_obj_t *mem = lv_label_create(scr,NULL);
  // lv_obj_set_style_local_text_font(mem,LV_OBJ_PART_MAIN,LV_STATE_DEFAULT,&lv_font_unscii_8);
  // lv_obj_set_style_local_text_color(mem,LV_OBJ_PART_MAIN,LV_STATE_DEFAULT,LV_COLOR_RED);
  // lv_label_set_text_fmt(mem,"%d",info.free_size);
  // lv_obj_align(mem,NULL,LV_ALIGN_IN_BOTTOM_RIGHT,0,0);

  //进入动画，进行切换
  obj_anim_in(scr, 100, 100, anim_in_ready_cb, NULL);
  return scr;
}

lv_obj_t *page_clock_event(RTC_Date *date)
{
  //    if()
}

void page_clock_exit(void *data)
{

  lv_task_del(task);
  //退出动画，进行切换
  obj_anim_out(scr, 100, 0, anim_out_ready_cb, NULL);
}

static void task_time_update(lv_task_t *t)
{
  static uint8_t hour, minute, second = 0;
  uint16_t voltage = app_batt_volatge();
  // Serial.println(voltage);
  second++;
  if (second == 120)
  {
    second = 0;
  }
  if (second == 0)
  {

    RTC_Date date = rtc.getDateTime();
    lv_label_set_text_fmt(label_time, "%02d%02d", date.hour, date.minute);
    lv_label_set_text_fmt(label_batt, "%d%%", app_batt_percent());
    lv_label_set_text_fmt(label_vol, "%dmV", app_batt_volatge());
  }

  if (led_second != NULL)
    lv_led_toggle(led_second);
}

static void anim_in_ready_cb(lv_anim_t *a)
{
  app_key_clean();
  manager_set_busy_state(false);
}

static void anim_out_ready_cb(lv_anim_t *a)
{
  lv_obj_del(scr);
}

//根据按键情况，对其进行不同操作处理
void page_clock_loop(void *data)
{
  KEY_TYPE key = app_key_get();
  switch (key)
  {
  case KEY0_SHORT:
    manager_switch(LEVEL_PREV);
    break;
  case KEY1_SHORT:
    manager_switch(LEVEL_NEXT);
    break;
  case KEY0_LONG:
    //  manager_switch(LEVEL_PARENT);
    break;
  case KEY1_LONG:
    if (manager.cur->child != NULL)
    {
      manager_switch(LEVEL_CHILD);
    }

    break;

  default:
    break;
  }
}