#include "page_manager.h"
#include "lvgl.h"
#include <Arduino.h>

QueueHandle_t Event_Queue;

page_manager_t manager;

void page_item_reset(page_t *page)
{
    memset(page, 0, sizeof(page_t));
}

// //parent的child只能有一个
// void page_item_set_parent_child(page_t *page_parent, page_t *page_child)
// {
//     page_parent->child = page_child;
//     page_child->parent = page_parent;
// }

// void page_item_set_prev_next(page_t *page, page_t *page_prev, page_t *page_next)
// {
//     page_parent->child = page_child;
//     page_child->parent = page_parent;
// }

void page_item_set_parent(page_t *page, page_t *parent)
{
    page->parent = parent;
}

void page_item_set_child(page_t *page, page_t *child)
{
    page->child = child;
}

void page_item_set_next(page_t *page, page_t *next)
{
    page->next = next;
}

void page_item_set_prev(page_t *page, page_t *prev)
{
    page->prev = prev;
}

void page_item_set_cb_enter(page_t *page, enter_cb_t cb)
{
    page->enter = cb;
}

void page_item_set_cb_exit(page_t *page, exit_cb_t cb)
{
    page->exit = cb;
}

void page_item_set_cb_loop(page_t *page, loop_cb_t cb)
{
    page->loop = cb;
}

void page_item_add_child(page_t *page, page_t *child, bool isEnd)
{
    if (page->child == NULL)
    {
        page->child = child;
    }
    child->index = page->child_num;
    child->parent = page;

    // 获取同一级的前一个选项
    if (child->index == 1)
    {
        child->prev = page->child;
        page->child->next = child;
    }
    page_t *page_tmp = page->child;
    if (child->index >= 2)
    {
        for (uint8_t i = 2; i < (page->child_num + 1); i++)
        {
            page_tmp = page_tmp->next;
        }

        child->prev = page_tmp;
        page_tmp->next = child;
    }

    page->child_num++;

    if (isEnd)
    {
        child->next = page->child;
        page->child->prev = child;
    }
}

void page_item_set_cb(page_t *page, enter_cb_t cb_enter, loop_cb_t cb_loop, exit_cb_t cb_exit)
{
    page->enter = cb_enter;
    page->loop = cb_loop;
    page->exit = cb_exit;
}

static void manager_loop(lv_task_t *task)
{
    event_type_t event;

    if (manager_get_busy_state() == false)
    {
        manager.cur->loop(NULL);
    }
    if (Event_Queue != 0)
    {
        // if(xQueueReceive(Key_Queue, &key_type,portMAX_DELAY));     //  10/portTICK_PERIOD_MS
        if (xQueueReceive(Event_Queue, &event, 0))
            ; //  10/portTICK_PERIOD_MS
        {
            //    Serial.println(event);
            switch (event)
            {
            case EVENT_ENTER:
                manager.cur->enter(&manager);
                break; //如果收到创建事件，进入页面，初始化相关内容
            // case EVENT_LOOP:manager.cur->loop(&manager);break;    //如果收到循环时间，进入处理阶段，主要是根据按键进行相应的处理
            case EVENT_EXIT:
                manager.old->exit(&manager);
                break; //如果收到退出时间，退出页面，进行清理工作
            default:
                break;
            }
        }
    }
}

void manager_switch(level_type_t level)
{

    event_type_t event;
    if (manager_get_busy_state() == false)
    {
        manager_set_busy_state(true);

        manager.old = manager.cur;
        if (Event_Queue != 0)
        {
            event = EVENT_EXIT;
            BaseType_t err = xQueueSend(Event_Queue, &event, 10);
            if (err = errQUEUE_FULL)
            {
                // 队列已满，数据发送失败
            }
        }
        switch (level)
        {
        case LEVEL_NEXT:
            manager.cur = manager.cur->next;
            break;
        case LEVEL_PREV:
            manager.cur = manager.cur->prev;
            break;
        case LEVEL_CHILD:
            manager.cur = manager.cur->child;
            break;
        case LEVEL_PARENT:
            manager.cur = manager.cur->parent;
            break;
        default:
            break;
        }

        if (Event_Queue != 0)
        {
            event = EVENT_ENTER;
            BaseType_t err = xQueueSend(Event_Queue, &event, 10);
            if (err = errQUEUE_FULL)
            {
                // 队列已满，数据发送失败
            }
        }
    }
}

void manager_switch_child(uint8_t index)
{
    if (index > (manager.cur->child_num - 1))
        return;
    event_type_t event;
    if (manager_get_busy_state() == false)
    {
        manager_set_busy_state(true);

        manager.old = manager.cur;
        if (Event_Queue != 0)
        {
            event = EVENT_EXIT;
            BaseType_t err = xQueueSend(Event_Queue, &event, 10);
            if (err = errQUEUE_FULL)
            {
                // 队列已满，数据发送失败
            }
        }
        manager.cur = manager.cur->child;
        for (uint8_t i = 1; i <= index; i++)
        {
            manager.cur = manager.cur->next;
        }

        if (Event_Queue != 0)
        {
            event = EVENT_ENTER;
            BaseType_t err = xQueueSend(Event_Queue, &event, 10);
            if (err = errQUEUE_FULL)
            {
                // 队列已满，数据发送失败
            }
        }
    }
}

void manager_set_busy_state(bool state)
{

    manager.busy = state;
}

bool manager_get_busy_state(void)
{
    return manager.busy;
}

static lv_task_t *task = NULL;

bool manager_loop_start(void)
{
    task = lv_task_create(manager_loop, 20, LV_TASK_PRIO_MID, NULL);
    if (task != NULL)
    {
        return true;
    }
    else
    {
        return false;
    }
}

void manager_loop_end(void)
{
    if (task != NULL)
    {
        lv_task_del(task);
    }
}

// 创建队列，储存相关事件
bool manager_init(void)
{
    //创建按键消息队列
    Event_Queue = xQueueCreate(10, sizeof(uint8_t));
    if (Event_Queue == 0)
    {
        // Queue was not created and must not be used.
        return false; //创建失败
    }
    return true;
}

void manager_deinit(void)
{
    if (Event_Queue != 0)
    {
        vQueueDelete(Event_Queue);
    }
}