#ifndef __PAGE_MANAGER_H
#define __PAGE_MANAGER_H
#include <Arduino.h>
 

typedef void *(*enter_cb_t)(void *data);
typedef void (*exit_cb_t)(void *data);
typedef void (*loop_cb_t)(void *data);

typedef struct _page_t
{
     
    enter_cb_t enter;
    exit_cb_t exit;
    loop_cb_t loop;
    struct _page_t *parent;
    struct _page_t *child;//只记录第一项
    struct _page_t *next;
    struct _page_t *prev;
    uint8_t child_num;
    uint8_t index;//属于同一级的第几个


}page_t;

typedef struct _page_manager_t
{
    page_t *old;
    page_t *cur;
    bool busy;
    void *user_data;

}page_manager_t;


typedef enum _event_type_t
{   
    EVENT_NONE=0,
    EVENT_ENTER ,
    EVENT_LOOP,
    EVENT_EXIT

}event_type_t;

// 
typedef enum _level_type_t
{
    LEVEL_NONE=0,
    LEVEL_PARENT,
    LEVEL_CHILD,
    LEVEL_NEXT,
    LEVEL_PREV

}level_type_t;


extern page_manager_t manager;

void page_item_reset(page_t *page);
void page_item_set_parent(page_t *page, page_t *parent);
void page_item_set_child(page_t *page, page_t *child);
void page_item_set_next(page_t *page, page_t *next);
void page_item_set_prev(page_t *page, page_t *prev);
void page_item_add_child(page_t *page, page_t *child, bool isEnd);


void page_item_set_cb_enter(page_t *page, enter_cb_t cb);
void page_item_set_cb_exit(page_t *page, exit_cb_t cb);
void page_item_set_cb_loop(page_t *page, loop_cb_t cb);

void page_item_set_cb(page_t *page, enter_cb_t cb_enter, loop_cb_t cb_loop, exit_cb_t cb_exit);

void manager_switch(level_type_t level);
void manager_switch_child(uint8_t index);
void manager_set_busy_state(bool state);
bool manager_get_busy_state(void);


bool manager_loop_start(void);
void manager_loop_end(void);

// 创建队列，储存相关事件
bool manager_init(void);

void manager_deinit(void);



#endif
