#include "page_motion.h"
#include "app/app_motion.h"
#include "app/app_key.h"
#include "app/app_anim.h"
#include "lvgl.h"
#include <Arduino.h>

static lv_obj_t *scr;
static uint32_t step_current;        //当时步数
static uint32_t step_remain;         //剩余步数
static uint32_t step_target = 10000; //目标步数

static void loader_anim_cb(void *arc, lv_anim_value_t v);
static void anim_in_ready_cb(lv_anim_t *a);

lv_obj_t *page_motion_enter(void *data)
{
  app_motion_get_step(&step_current);
  // step_current += 2800;//test

  scr = lv_obj_create(lv_scr_act(), NULL);
  lv_obj_clean_style_list(scr, LV_OBJ_PART_MAIN);
  lv_obj_set_style_local_bg_opa(scr, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, LV_OPA_COVER);
  lv_obj_set_style_local_bg_color(scr, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, LV_COLOR_BLACK);
  lv_obj_set_style_local_border_color(scr, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, LV_COLOR_BLACK);
  lv_obj_set_style_local_radius(scr, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, 0);
  lv_obj_set_size(scr, LV_HOR_RES, LV_VER_RES);
  lv_obj_set_pos(scr, 0, 0);

  lv_obj_t *arc = lv_arc_create(scr, NULL);
  lv_obj_set_style_local_bg_color(arc, LV_ARC_PART_BG, LV_STATE_DEFAULT, LV_COLOR_BLACK);
  lv_obj_set_style_local_border_color(arc, LV_ARC_PART_BG, LV_STATE_DEFAULT, LV_COLOR_MAKE(0x08, 0x08, 0x08));
  lv_obj_set_style_local_radius(arc, LV_ARC_PART_BG, LV_STATE_DEFAULT, LV_RADIUS_CIRCLE);
  lv_obj_set_style_local_border_width(arc, LV_ARC_PART_BG, LV_STATE_DEFAULT, 6);
  // lv_obj_set_style_local_line_width(arc,LV_ARC_PART_BG,LV_STATE_DEFAULT,2);
  lv_obj_set_style_local_pad_bottom(arc, LV_ARC_PART_BG, LV_STATE_DEFAULT, 0);
  lv_obj_set_style_local_pad_top(arc, LV_ARC_PART_BG, LV_STATE_DEFAULT, 0);
  lv_obj_set_style_local_pad_left(arc, LV_ARC_PART_BG, LV_STATE_DEFAULT, 0);
  lv_obj_set_style_local_pad_right(arc, LV_ARC_PART_BG, LV_STATE_DEFAULT, 0);
  // lv_obj_set_style_local_pad_inner(arc,LV_ARC_PART_BG,LV_STATE_DEFAULT,5);
  lv_obj_set_style_local_line_width(arc, LV_ARC_PART_INDIC, LV_STATE_DEFAULT, 6);
  lv_obj_set_style_local_line_color(arc, LV_ARC_PART_INDIC, LV_STATE_DEFAULT, LV_COLOR_GREEN);

  lv_arc_set_bg_angles(arc, 0, 0);
  lv_arc_set_start_angle(arc, 270);
  lv_obj_set_size(arc, 80, 80);
  lv_obj_align(arc, NULL, LV_ALIGN_IN_LEFT_MID, 0, 0);

  // randomSeed(millis());
  // step_current = random(0,step_target);
  step_remain = (step_target >= step_current) ? (step_target - step_current) : 0;
  uint8_t percent = (step_target >= step_current) ? (step_current * 100 / step_target) : 100;

  lv_obj_t *label_steps_current = lv_label_create(scr, NULL);
  lv_obj_set_style_local_text_color(label_steps_current, LV_LABEL_PART_MAIN, LV_STATE_DEFAULT, LV_COLOR_GREEN);
  lv_label_set_text_fmt(label_steps_current, LV_SYMBOL_OK "%5d", step_current);

  lv_obj_t *label_steps_remain = lv_label_create(scr, NULL);
  lv_label_set_text_fmt(label_steps_remain, LV_SYMBOL_CLOSE "%5d", step_remain);
  lv_obj_set_style_local_text_color(label_steps_remain, LV_LABEL_PART_MAIN, LV_STATE_DEFAULT, LV_COLOR_RED);
  lv_label_set_recolor(label_steps_remain, true);

  lv_obj_t *label_steps_target = lv_label_create(scr, NULL);
  lv_label_set_text_fmt(label_steps_target, LV_SYMBOL_LOOP "%5d", step_target);

  lv_obj_align(label_steps_current, NULL, LV_ALIGN_IN_TOP_RIGHT, 0, 10);
  lv_obj_align(label_steps_remain, NULL, LV_ALIGN_IN_RIGHT_MID, 0, 0);
  lv_obj_align(label_steps_target, NULL, LV_ALIGN_IN_BOTTOM_RIGHT, 0, -10);

  // lv_mem_monitor_t info;
  // lv_mem_monitor(&info);
  // lv_obj_t *mem = lv_label_create(scr,NULL);
  // lv_obj_set_style_local_text_font(mem,LV_OBJ_PART_MAIN,LV_STATE_DEFAULT,&lv_font_unscii_8);
  // lv_obj_set_style_local_text_color(mem,LV_OBJ_PART_MAIN,LV_STATE_DEFAULT,LV_COLOR_RED);
  // lv_label_set_text_fmt(mem,"%d",info.free_size);
  // lv_obj_align(mem,NULL,LV_ALIGN_IN_BOTTOM_RIGHT,0,0);

  lv_anim_t a;
  lv_anim_init(&a);

  lv_anim_set_exec_cb(&a, loader_anim_cb);
  // lv_anim_set_ready_cb(&a, end_cb);
  lv_anim_set_values(&a, 0, (percent));
  lv_anim_set_time(&a, percent * 1000 / 100); //1s 100%
  // lv_anim_set_delay(&a,400);
  lv_anim_set_var(&a, arc);
  lv_anim_start(&a);

  obj_anim_in(label_steps_current, 100, 200, NULL, NULL);
  obj_anim_in(label_steps_remain, 100, 300, NULL, NULL);
  obj_anim_in(label_steps_target, 100, 400, NULL, NULL);

  //进入动画，进行切换
  obj_anim_in(scr, 100, 100, anim_in_ready_cb, NULL);
  return scr;
}

static void anim_in_ready_cb(lv_anim_t *a)
{
  app_key_clean();
  manager_set_busy_state(false);
}
static void loader_anim_cb(void *arc, lv_anim_value_t v)
{
  if (v > 100)
    v = 100;
  lv_arc_set_end_angle((lv_obj_t *)arc, v * 360 / 100 + 270);

  static char buf[32];
  lv_snprintf(buf, sizeof(buf), "%d %%", v);
  lv_obj_set_style_local_value_str((lv_obj_t *)arc, LV_ARC_PART_BG, LV_STATE_DEFAULT, buf);
  lv_obj_set_style_local_value_color((lv_obj_t *)arc, LV_ARC_PART_BG, LV_STATE_DEFAULT, LV_COLOR_GREEN);
}

static void anim_out_ready_cb(lv_anim_t *a)
{
  lv_obj_del(scr);
}
void page_motion_exit(void *data)
{

  //   lv_task_del(task);
  //退出动画，进行切换
  obj_anim_out(scr, 100, 0, anim_out_ready_cb, NULL);
}

//根据按键情况，对其进行不同操作处理
void page_motion_loop(void *data)
{
  KEY_TYPE key = app_key_get();
  switch (key)
  {
  case KEY0_SHORT:
    manager_switch(LEVEL_PREV);
    break;
  case KEY1_SHORT:
    manager_switch(LEVEL_NEXT);
    break;
  case KEY0_LONG:
    //  manager_switch(LEVEL_PARENT);
    break;
  case KEY1_LONG:
    manager_switch(LEVEL_CHILD);
    break;

  default:
    break;
  }
}

//重置每天的计步值
static void reset_stepcount(void)
{
}