#include "page_mouse.h"
#include "page/page_manager.h"

#include "app/app_anim.h"
#include "app/app_key.h"
#include "app/app_motion.h"
#include "app/app_ble.h"
#define MOVE_STEP 10
static lv_obj_t *scr;
static lv_obj_t *label;
static lv_task_t *task;
void task_motion(struct _lv_task_t *);

static void anim_in_ready_cb(lv_anim_t *a)
{
  app_key_clean();
  manager_set_busy_state(false);
}

static void anim_out_ready_cb(lv_anim_t *a)
{
  lv_obj_del(scr);
}

lv_obj_t *page_mouse_enter(void *data)
{

  scr = lv_obj_create(lv_scr_act(), NULL);
  lv_obj_clean_style_list(scr, LV_OBJ_PART_MAIN);
  lv_obj_set_style_local_bg_opa(scr, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, LV_OPA_COVER);
  lv_obj_set_style_local_bg_color(scr, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, LV_COLOR_BLACK);
  lv_obj_set_style_local_border_color(scr, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, LV_COLOR_BLACK);
  lv_obj_set_style_local_radius(scr, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, 0);
  lv_obj_set_size(scr, LV_HOR_RES, LV_VER_RES);
  lv_obj_set_pos(scr, 0, 0);

  label = lv_label_create(scr, NULL);
  lv_label_set_text(label, "x:\ny:\nz");
  lv_obj_align(label, scr, LV_ALIGN_IN_TOP_LEFT, 10, 10);

  //进入动画，进行切换
  obj_anim_in(scr, 100, 100, anim_in_ready_cb, NULL);

  // task = lv_task_create(task_motion, 100, LV_TASK_PRIO_MID, NULL);
  if (task == NULL)
  {
    // tft.setCursor(0, 0);
    // tft.setTextColor(TFT_RED, TFT_BLACK);
    // tft.println("task create fail");
  }

  lv_mem_monitor_t info;
  lv_mem_monitor(&info);
  lv_obj_t *mem = lv_label_create(scr, NULL);
  lv_obj_set_style_local_text_font(mem, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, &lv_font_unscii_8);
  lv_obj_set_style_local_text_color(mem, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, LV_COLOR_RED);
  lv_label_set_text_fmt(mem, "%d", info.free_size);
  lv_obj_align(mem, NULL, LV_ALIGN_IN_BOTTOM_RIGHT, 0, 0);

  return scr;
}
void task_motion(struct _lv_task_t *)
{
  Angle_t angle;
  bool res;

  bool state_conn = app_ble_get_state_conn();
  if (state_conn)
  {
    app_motion_get_angle(&angle);

    lv_label_set_text_fmt(label, "x:%d\ny:%d\nz:%d", angle.X_Angle / 1, angle.Y_Angle / 1, angle.Z_Angle / 1);

    if (angle.X_Angle > 20.0)
      blehid.mouseMove(-MOVE_STEP, 0);

    if (angle.X_Angle < -20.0)
      blehid.mouseMove(MOVE_STEP, 0);

    if (angle.Y_Angle > 20.0)
      blehid.mouseMove(0, -MOVE_STEP);

    if (angle.Y_Angle < -20.0)
      blehid.mouseMove(0, MOVE_STEP);
  }
}

//根据按键情况，对其进行不同操作处理
void page_mouse_loop(void *data)
{

  task_motion(NULL);

  bool state_conn = app_ble_get_state_conn();
  KEY_TYPE key = app_key_get();
  switch (key)
  {
  case KEY0_SHORT:
    // manager_switch(LEVEL_PREV);
    if (state_conn)
    {
      blehid.mouseButtonPress(MOUSE_BUTTON_RIGHT);
      blehid.mouseButtonRelease();
    }
    break;
  case KEY1_SHORT:
    // manager_switch(LEVEL_NEXT);
    if (state_conn)
    {
      blehid.mouseButtonPress(MOUSE_BUTTON_LEFT);
      blehid.mouseButtonRelease();
    }

    break;
  case KEY0_LONG:
    manager_switch(LEVEL_PARENT);

    break;
  case KEY1_LONG:
    // manager_switch(LEVEL_CHILD);

    break;

  default:
    break;
  }
}

void page_mouse_exit(void *data)
{

  lv_task_del(task);
  //退出动画，进行切换
  obj_anim_out(scr, 100, 0, anim_out_ready_cb, NULL);
}
