#include "page_time_set.h"

#include <Arduino.h>

#include "app/app_clock.h"
#include "board/board.h"
#include "app/app_key.h"
#include "app/app_anim.h"
#include "page/page_manager.h"
static lv_obj_t *scr;
static lv_obj_t *roller[3];
static uint8_t item = 0;

static void anim_in_ready_cb(lv_anim_t *a)
{
    app_key_clean();
    manager_set_busy_state(false);
}

static void anim_out_ready_cb(lv_anim_t *a)
{
    lv_obj_del(scr);
}

lv_obj_t *page_time_set_enter(void *data)
{

    date = rtc.getDateTime();

    scr = lv_obj_create(lv_scr_act(), NULL);
    lv_obj_clean_style_list(scr, LV_OBJ_PART_MAIN);
    lv_obj_set_style_local_bg_opa(scr, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, LV_OPA_COVER);
    lv_obj_set_style_local_bg_color(scr, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, LV_COLOR_BLACK);
    lv_obj_set_style_local_border_color(scr, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, LV_COLOR_BLACK);
    lv_obj_set_style_local_radius(scr, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, 0);
    lv_obj_set_size(scr, LV_HOR_RES, LV_VER_RES);
    lv_obj_set_pos(scr, 0, 0);

    roller[0] = lv_roller_create(scr, NULL);
    lv_obj_set_style_local_text_font(roller[0], LV_ROLLER_PART_BG, LV_STATE_DEFAULT, &lv_font_unscii_8);
    lv_obj_set_style_local_text_font(roller[0], LV_ROLLER_PART_SELECTED, LV_STATE_DEFAULT, &lv_font_unscii_8);
    lv_obj_set_style_local_bg_color(roller[0], LV_ROLLER_PART_SELECTED, LV_STATE_DISABLED, LV_COLOR_GRAY);
    lv_roller_set_options(roller[0],
                          "00\n"
                          "01\n"
                          "02\n"
                          "03\n"
                          "04\n"
                          "05\n"
                          "06\n"
                          "07\n"
                          "08\n"
                          "09\n"
                          "10\n"
                          "11\n"
                          "12\n"
                          "13\n"
                          "14\n"
                          "15\n"
                          "16\n"
                          "17\n"
                          "18\n"
                          "19\n"
                          "20\n"
                          "21\n"
                          "22\n"
                          "23",
                          LV_ROLLER_MODE_INIFINITE);

    lv_roller_set_visible_row_count(roller[0], 3);
    lv_roller_set_auto_fit(roller[0], true);
    lv_roller_set_selected(roller[0], date.hour, LV_ANIM_ON);

    roller[1] = lv_roller_create(scr, NULL);
    lv_obj_set_style_local_text_font(roller[1], LV_ROLLER_PART_BG, LV_STATE_DEFAULT, &lv_font_unscii_8);
    lv_obj_set_style_local_text_font(roller[1], LV_ROLLER_PART_SELECTED, LV_STATE_DEFAULT, &lv_font_unscii_8);
    lv_obj_set_style_local_bg_color(roller[1], LV_ROLLER_PART_SELECTED, LV_STATE_DISABLED, LV_COLOR_GRAY);
    lv_roller_set_options(roller[1],
                          "00\n"
                          "01\n"
                          "02\n"
                          "03\n"
                          "04\n"
                          "05\n"
                          "06\n"
                          "07\n"
                          "08\n"
                          "09\n"
                          "10\n"
                          "11\n"
                          "12\n"
                          "13\n"
                          "14\n"
                          "15\n"
                          "16\n"
                          "17\n"
                          "18\n"
                          "19\n"
                          "20\n"
                          "21\n"
                          "22\n"
                          "23\n"
                          "24\n"
                          "25\n"
                          "26\n"
                          "27\n"
                          "28\n"
                          "29\n"
                          "30\n"
                          "31\n"
                          "32\n"
                          "33\n"
                          "34\n"
                          "35\n"
                          "36\n"
                          "37\n"
                          "38\n"
                          "39\n"
                          "40\n"
                          "41\n"
                          "42\n"
                          "43\n"
                          "44\n"
                          "45\n"
                          "46\n"
                          "47\n"
                          "48\n"
                          "49\n"
                          "50\n"
                          "51\n"
                          "52\n"
                          "53\n"
                          "54\n"
                          "55\n"
                          "56\n"
                          "57\n"
                          "58\n"
                          "59",
                          LV_ROLLER_MODE_INIFINITE);

    lv_roller_set_visible_row_count(roller[1], 3);
    lv_roller_set_auto_fit(roller[1], true);
    lv_roller_set_selected(roller[1], date.minute, LV_ANIM_ON);

    // lv_obj_set_event_cb(roller[1], event_handler);

    roller[2] = lv_roller_create(scr, NULL);
    lv_obj_set_style_local_text_font(roller[2], LV_ROLLER_PART_BG, LV_STATE_DEFAULT, &lv_font_unscii_8);
    lv_obj_set_style_local_text_font(roller[2], LV_ROLLER_PART_SELECTED, LV_STATE_DEFAULT, &lv_font_unscii_8);
    lv_obj_set_style_local_bg_color(roller[2], LV_ROLLER_PART_SELECTED, LV_STATE_DISABLED, LV_COLOR_GRAY);
    lv_roller_set_options(roller[2],
                          "00\n"
                          "01\n"
                          "02\n"
                          "03\n"
                          "04\n"
                          "05\n"
                          "06\n"
                          "07\n"
                          "08\n"
                          "09\n"
                          "10\n"
                          "11\n"
                          "12\n"
                          "13\n"
                          "14\n"
                          "15\n"
                          "16\n"
                          "17\n"
                          "18\n"
                          "19\n"
                          "20\n"
                          "21\n"
                          "22\n"
                          "23\n"
                          "24\n"
                          "25\n"
                          "26\n"
                          "27\n"
                          "28\n"
                          "29\n"
                          "30\n"
                          "31\n"
                          "32\n"
                          "33\n"
                          "34\n"
                          "35\n"
                          "36\n"
                          "37\n"
                          "38\n"
                          "39\n"
                          "40\n"
                          "41\n"
                          "42\n"
                          "43\n"
                          "44\n"
                          "45\n"
                          "46\n"
                          "47\n"
                          "48\n"
                          "49\n"
                          "50\n"
                          "51\n"
                          "52\n"
                          "53\n"
                          "54\n"
                          "55\n"
                          "56\n"
                          "57\n"
                          "58\n"
                          "59",
                          LV_ROLLER_MODE_INIFINITE);
    lv_roller_set_visible_row_count(roller[2], 3);
    lv_roller_set_auto_fit(roller[2], true);
    lv_roller_set_selected(roller[2], date.second, LV_ANIM_ON);

    lv_obj_set_style_local_bg_color(roller[0], LV_ROLLER_PART_SELECTED, LV_STATE_DEFAULT, LV_COLOR_GREEN);
    lv_obj_set_style_local_bg_color(roller[1], LV_ROLLER_PART_SELECTED, LV_STATE_DEFAULT, LV_COLOR_GRAY);
    lv_obj_set_style_local_bg_color(roller[2], LV_ROLLER_PART_SELECTED, LV_STATE_DEFAULT, LV_COLOR_GRAY);

    lv_obj_align(roller[0], NULL, LV_ALIGN_IN_LEFT_MID, 0, 0);
    lv_obj_align(roller[1], roller[0], LV_ALIGN_OUT_RIGHT_MID, 3, 0);
    lv_obj_align(roller[2], roller[1], LV_ALIGN_OUT_RIGHT_MID, 3, 0);

    // lv_mem_monitor_t info;
    // lv_mem_monitor(&info);
    // lv_obj_t *mem = lv_label_create(scr, NULL);
    // lv_obj_set_style_local_text_font(mem, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, &lv_font_unscii_8);
    // lv_obj_set_style_local_text_color(mem, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, LV_COLOR_RED);
    // lv_label_set_text_fmt(mem, "%d", info.free_size);
    // lv_obj_align(mem, scr, LV_ALIGN_IN_BOTTOM_RIGHT, 0, 0);

    //进入动画，进行切换
    obj_anim_in(scr, 100, 100, anim_in_ready_cb, NULL);
    return scr;
}
static void get_str(char *buffer);
//根据按键情况，对其进行不同操作处理
void page_time_set_loop(void *data)
{
    KEY_TYPE key = app_key_get();
    uint16_t id = lv_roller_get_selected(roller[item]);
    uint16_t cnt = lv_roller_get_option_cnt(roller[item]);

    switch (key)
    {
    case KEY0_SHORT:
        id--;
        if (id == 0)
        {
            id = cnt - 1;
        }
        lv_roller_set_selected(roller[item], id, LV_ANIM_ON);
        //   manager_switch(LEVEL_PREV);
        break;
    case KEY1_SHORT:
        //   manager_switch(LEVEL_NEXT);
        id++;
        if (id == cnt)
        {
            id = 0;
        }
        lv_roller_set_selected(roller[item], id, LV_ANIM_ON);

        break;
    case KEY0_LONG:
        //获取相关时间
        char buffer[10];
        char newstr[20];
        get_str(buffer);
        sprintf(newstr, "%04d%02d%02d%s", date.year, date.month, date.day, buffer);
        app_clock_set_time_str(newstr);
        //   Serial.println(newstr);

        manager_switch(LEVEL_PARENT);
        break;
    case KEY1_LONG:
        // manager_switch(LEVEL_CHILD);
        item++;
        if (item == 3)
        {
            item = 0;
        }
        for (uint8_t i = 0; i < 3; i++)
        {
            if (i == item)
            {
                lv_obj_set_style_local_bg_color(roller[item], LV_ROLLER_PART_SELECTED, LV_STATE_DEFAULT, LV_COLOR_GREEN);
            }
            else
            {
                lv_obj_set_style_local_bg_color(roller[i], LV_ROLLER_PART_SELECTED, LV_STATE_DEFAULT, LV_COLOR_GRAY);
            }
        }

        break;

    default:
        break;
    }
}

void page_time_set_exit(void *data)
{

    // lv_task_del(task);
    //退出动画，进行切换
    obj_anim_out(scr, 100, 0, anim_out_ready_cb, NULL);
}

static void get_str(char *buffer)
{
    char buffer0[3];
    char buffer1[3];
    char buffer2[3];

    lv_roller_get_selected_str(roller[0], buffer0, 3);
    lv_roller_get_selected_str(roller[1], buffer1, 3);
    lv_roller_get_selected_str(roller[2], buffer2, 3);
    // Serial.println(buffer0);
    // Serial.println(buffer1);
    // Serial.println(buffer2);
    snprintf(buffer, 10, "%s%s%s", buffer0, buffer1, buffer2);
}